/* eslint-disable no-magic-numbers */
import { render, screen } from '@testing-library/react';
import {
  InitialStoreState,
  getReduxWrapperWithStore,
} from '@/utils/testing/getReduxWrapperWithStore';
import { StoreType } from '@/redux/store';
import { drugsFixture } from '@/models/fixtures/drugFixtures';
import { drawerSearchDrugsStepTwoFixture } from '@/redux/drawer/drawerFixture';
import { PinsList } from './PinsList.component';

const PINS_LIST = drugsFixture.map(drug => ({
  id: drug.id,
  name: drug.name,
  data: drug,
}));

const renderComponent = (initialStoreState: InitialStoreState = {}): { store: StoreType } => {
  const { Wrapper, store } = getReduxWrapperWithStore(initialStoreState);

  return (
    render(
      <Wrapper>
        <PinsList pinsList={PINS_LIST} type="drugs" />
      </Wrapper>,
    ),
    {
      store,
    }
  );
};

describe('PinsList component', () => {
  it('should render list of pins', () => {
    renderComponent();

    const fristDrugName = drugsFixture[0].name;
    const secondDrugName = drugsFixture[1].name;

    expect(screen.getByText(fristDrugName)).toBeInTheDocument();
    expect(screen.getByText(secondDrugName)).toBeInTheDocument();
  });
  it('should navigate to details step on pin click', () => {
    const { store } = renderComponent({ drawer: drawerSearchDrugsStepTwoFixture });

    const firstPin = screen.getAllByRole('button')[0];
    firstPin.click();

    const {
      drawer: {
        searchDrawerState: { currentStep, stepType, selectedValue },
      },
    } = store.getState();
    const drug = drugsFixture[0];

    expect(currentStep).toBe(3);
    expect(stepType).toBe('drugs');
    expect(selectedValue).toEqual(drug);
  });
});
