/* eslint-disable no-magic-numbers */
import { OPTIONS } from '@/constants/map';
import { mapDataPositionSelector, mapDataSizeSelector } from '@/redux/map/map.selectors';
import { Point } from '@/types/map';
import { usePointToProjection } from '@/utils/map/usePointToProjection';
import { View } from 'ol';
import BaseLayer from 'ol/layer/Base';
import TileLayer from 'ol/layer/Tile';
import { XYZ } from 'ol/source';
import { useMemo } from 'react';
import { useSelector } from 'react-redux';

interface UseOlMapConfigResult {
  view: View;
  layers: BaseLayer[];
}

export const useOlMapConfig = (): UseOlMapConfigResult => {
  const mapPosition = useSelector(mapDataPositionSelector);
  const mapSize = useSelector(mapDataSizeSelector);
  const pointToProjection = usePointToProjection();

  const center = useMemo(() => {
    const centerPoint: Point = {
      x: mapPosition.x,
      y: mapPosition.y,
    };

    return pointToProjection(centerPoint);
  }, [mapPosition, pointToProjection]);

  const view = useMemo(
    () =>
      new View({
        center,
        zoom: mapPosition.z,
        showFullExtent: OPTIONS.showFullExtent,
      }),
    [center, mapPosition],
  );

  const tileLayer = useMemo(
    () =>
      new TileLayer({
        visible: true,
        source: new XYZ({
          url: 'https://pdmap.uni.lu/map_images/9d4911bdeeea752f076e57a91d9b1f45/_nested0/{z}/{x}/{y}.PNG',
          // TODO: build url from data in redux
          maxZoom: mapSize.maxZoom,
          minZoom: mapSize.minZoom,
          tileSize: mapSize.tileSize,
          wrapX: OPTIONS.wrapXInTileLayer,
        }),
      }),
    [mapSize],
  );

  return {
    view,
    layers: [tileLayer],
  };
};
