import { createSelector } from '@reduxjs/toolkit';
import { backgroundsSelector } from '../backgrounds/background.selectors';
import { modelsSelector } from '../models/models.selectors';
import { overlaysSelector } from '../overlays/overlays.selectors';
import { projectSelector } from '../project/project.selectors';

export const mapLoadingFirstStageInitializedSelector = createSelector(
  projectSelector,
  project => project.loading !== 'idle',
);

export const mapLoadingFirstStageCompletedSelector = createSelector(
  projectSelector,
  project => project.loading === 'succeeded',
);

export const mapLoadingSecondStageInitializedSelector = createSelector(
  backgroundsSelector,
  modelsSelector,
  overlaysSelector,
  (backgrounds, models, overlays) =>
    [backgrounds.loading, models.loading, overlays.loading].every(loading => loading !== 'idle'),
);

export const mapLoadingSecondStageCompletedSelector = createSelector(
  backgroundsSelector,
  modelsSelector,
  overlaysSelector,
  (backgrounds, models, overlays) =>
    [backgrounds.loading, models.loading, overlays.loading].every(
      loading => loading === 'succeeded',
    ),
);

export const mapLoadingAllStagesCompletedSelector = createSelector(
  mapLoadingFirstStageCompletedSelector,
  mapLoadingSecondStageCompletedSelector,
  (firstStageCompleted, secondStageCompleted) =>
    [firstStageCompleted, secondStageCompleted].every(completed => completed === true),
);
