import { RootState, StoreType, reducers } from '@/redux/store';
import { configureStore } from '@reduxjs/toolkit';
import { Provider } from 'react-redux';

interface WrapperProps {
  children: React.ReactNode;
}

export type InitialStoreState = Partial<RootState>;

type GetReduxWrapperUsingSliceReducer = (initialState?: InitialStoreState) => {
  Wrapper: ({ children }: WrapperProps) => JSX.Element;
  store: StoreType;
};

export const getReduxWrapperWithStore: GetReduxWrapperUsingSliceReducer = (
  preloadedState: InitialStoreState = {},
) => {
  const testStore = configureStore({
    reducer: reducers,
    preloadedState,
  });

  const Wrapper = ({ children }: WrapperProps): JSX.Element => (
    <Provider store={testStore}>{children}</Provider>
  );

  return { Wrapper, store: testStore };
};
